import com.huawei.sis.bean.AuthInfo;
import com.huawei.sis.bean.SisConfig;
import com.huawei.sis.bean.SisConstant;
import com.huawei.sis.bean.request.AsrCustomShortRequest;
import com.huawei.sis.bean.response.AsrCustomShortResponse;
import com.huawei.sis.client.AsrCustomizationClient;
import com.huawei.sis.exception.SisException;
import com.huawei.sis.util.IOUtils;
import com.huawei.sis.util.JsonUtils;

/**
 * 一句话识别Demo
 *
 * Copyright 2021 Huawei Technologies Co.,Ltd.
 */
public class SasrDemo {
  private String ak = "";
  private String sk = "";
  private String region = "";    // 区域，如cn-north-1、cn-north-4
  private String projectId = ""; // 项目id，在我的凭证查看。参考https://support.huaweicloud.com/api-sis/sis_03_0008.html

  /**
   *   todo 请正确填写音频格式和模型属性字符串
   *   1. 音频格式一定要相匹配.
   *      例如wav音频，格式是wav。具体参考api文档。
   *      例如音频是pcm格式，并且采样率为8k，则格式填写pcm8k16bit。
   *      如果返回audio_format is invalid 说明该文件格式不支持。具体支持哪些音频格式，需要参考api文档。
   *
   *   2. 音频采样率要与属性字符串的采样率要匹配。
   *      例如格式选择pcm16k16bit，属性字符串却选择chinese_8k_common, 则会返回'audio_format' is not match model
   *      例如wav本身是16k采样率，属性选择chinese_8k_common, 同样会返回'audio_format' is not match model
   */
  private String path = "";             // 文件路径，需具体到文件，如D:/test.wav
  private String pathAudioFormat = "";  // 文件格式，如wav等， 支持格式详见api文档
  private String pathProperty = "";     // 属性字符串，language_sampleRate_domain, 如chinese_8k_common, 详见api文档


  /**
   * 设置一句话识别参数，所有参数均有默认值，不配置也可使用
   *
   * @param request 一句话识别请求
   */
  private void setShortParameter(AsrCustomShortRequest request) {

    // 设置是否添加标点，默认是no
    request.setAddPunc("yes");
    // 设置热词id，详见api文档，若热词id不存在，则会报错
    // request.setVocabularyId("");
    // 设置是否将音频中数字转写为阿拉伯数字，yes or no，默认yes
    request.setDigitNorm("no");
    // 设置是否需要word_info，yes or no, 默认no
    request.setNeedWordInfo("no");

  }

  /**
   * 定义config，所有参数可选，设置超时时间等。
   *
   * @return SisConfig
   */
  private SisConfig getConfig() {
    SisConfig config = new SisConfig();
    // 设置连接超时，默认10000ms
    config.setConnectionTimeout(SisConstant.DEFAULT_CONNECTION_TIMEOUT);
    // 设置读取超时，默认10000ms
    config.setReadTimeout(SisConstant.DEFAULT_READ_TIMEOUT);
    // 设置代理, 一定要确保代理可用才启动此设置。 代理初始化也可用不加密的代理，new ProxyHostInfo(host, port);
    // ProxyHostInfo proxy = new ProxyHostInfo(host, port, username, password);
    // config.setProxy(proxy);
    return config;
  }

  /**
   * 一句话识别demo。
   */
  private void sasrDemo() {
    try {

      // 1. 初始化AsrCustomizationClient
      // 定义authInfo，根据ak，sk，region，projectId
      AuthInfo authInfo = new AuthInfo(ak, sk, region, projectId);
      // 设置config，主要与超时有关
      SisConfig config = getConfig();
      // 根据authInfo和config，构造AsrCustomizationClient
      AsrCustomizationClient asr = new AsrCustomizationClient(authInfo, config);

      // 2. 配置请求
      String data = IOUtils.getEncodeDataByPath(path);
      AsrCustomShortRequest request = new AsrCustomShortRequest(data, pathAudioFormat, pathProperty);
      // 设置请求参数，所有参数均为可选
      setShortParameter(request);

      // 3. 发送请求，获取响应。具体结果可通过response.getXX获取。
      AsrCustomShortResponse response = asr.getAsrShortResponse(request);
      System.out.println(JsonUtils.obj2Str(response, true));
    } catch (SisException e) {
      e.printStackTrace();
      System.out.println("error_code:" + e.getErrorCode() + "\nerror_msg:" + e.getErrorMsg());
    }
  }


  public static void main(String[] args) {
    SasrDemo demo = new SasrDemo();
    demo.sasrDemo();

  }
}
